@@ -19,7 +19,7 @@
 #ifndef vte_vte_h_included
 #define vte_vte_h_included
 
-#ident "$Id: vte.h,v 1.51 2002/10/15 02:22:37 nalin Exp $"
+#ident "$Id: vte.h,v 1.54 2002/10/30 20:03:22 nalin Exp $"
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -86,8 +86,9 @@ struct _VteTerminalClass {
 	guint resize_window_signal;
 	guint move_window_signal;
 	guint status_line_changed_signal;
-
 	guint commit_signal;
+
+	gpointer reserved1;
 	gpointer reserved2;
 	gpointer reserved3;
 	gpointer reserved4;
@@ -133,15 +134,11 @@ GtkType vte_terminal_erase_binding_get_type(void);
 /* You can get by with just these two functions. */
 GtkWidget *vte_terminal_new(void);
 pid_t vte_terminal_fork_command(VteTerminal *terminal,
-			        const char *command, char **argv, char **envv);
-
-/* If you need the session logged, try this instead. */
-pid_t vte_terminal_fork_logged_command(VteTerminal *terminal,
-				       const char *command, char **argv,
-				       char **envv,
-				       gboolean lastlog,
-				       gboolean utmp,
-				       gboolean wtmp);
+				const char *command, char **argv,
+				char **envv, const char *directory,
+				gboolean lastlog,
+				gboolean utmp,
+				gboolean wtmp);
 
 /* Send data to the terminal to display, or to the terminal's forked command
  * to handle in some way.  If it's 'cat', they should be the same. */
@@ -170,6 +167,8 @@ void vte_terminal_set_scroll_on_keystroke(VteTerminal *terminal,
 					  gboolean scroll);
 
 /* Set the color scheme. */
+void vte_terminal_set_color_dim(VteTerminal *terminal,
+				 const GdkColor *bold);
 void vte_terminal_set_color_bold(VteTerminal *terminal,
 				 const GdkColor *bold);
 void vte_terminal_set_color_foreground(VteTerminal *terminal,
@@ -243,14 +242,18 @@ void vte_terminal_reset(VteTerminal *terminal, gboolean full,
 char *vte_terminal_get_text(VteTerminal *terminal,
 			    gboolean(*is_selected)(VteTerminal *terminal,
 						   glong column,
-						   glong row),
+						   glong row,
+						   gpointer data),
+			    gpointer data,
 			    GArray *attributes);
 char *vte_terminal_get_text_range(VteTerminal *terminal,
 				  glong start_row, glong start_col,
 				  glong end_row, glong end_col,
 				  gboolean(*is_selected)(VteTerminal *terminal,
 							 glong column,
-							 glong row),
+							 glong row,
+							 gpointer data),
+				  gpointer data,
 				  GArray *attributes);
 void vte_terminal_get_cursor_position(VteTerminal *terminal,
 				      glong *column, glong *row);
@@ -261,6 +264,8 @@ void vte_terminal_match_clear_all(VteTerminal *terminal);
 /* Add a matching expression, returning the tag the widget assigns to that
  * expression. */
 int vte_terminal_match_add(VteTerminal *terminal, const char *match);
+/* Remove a matching expression by tag. */
+void vte_terminal_match_remove(VteTerminal *terminal, int tag);
 
 /* Check if a given cell on the screen contains part of a matched string.  If
  * it does, return the string, and store the match tag in the optional tag
